@@ -1,12 +1,11 @@
+use fxhash::FxHashMap;
 use std::any::{Any, TypeId};
-use std::collections::HashMap;
-use std::hash::{BuildHasherDefault, Hasher};
 
 pub(crate) type AnyObject = Box<dyn Any + Send + Sync>;
 
 #[derive(Debug, Default)]
 pub struct TypeMap {
-    inner: HashMap<TypeId, AnyObject, BuildHasherDefault<IdentHash>>,
+    inner: FxHashMap<TypeId, AnyObject>,
 }
 
 impl TypeMap {
@@ -44,35 +43,3 @@ impl TypeMap {
         self.inner.extend(other.inner)
     }
 }
-
-pub struct IdentHash(u64);
-
-impl Hasher for IdentHash {
-    #[inline]
-    fn finish(&self) -> u64 {
-        self.0
-    }
-
-    #[inline]
-    fn write(&mut self, bytes: &[u8]) {
-        for byte in bytes {
-            self.write_u8(*byte);
-        }
-    }
-
-    #[inline]
-    fn write_u8(&mut self, i: u8) {
-        self.0 = (self.0 << 8) | (i as u64);
-    }
-
-    #[inline]
-    fn write_u64(&mut self, i: u64) {
-        self.0 = i;
-    }
-}
-
-impl Default for IdentHash {
-    fn default() -> IdentHash {
-        IdentHash(0)
-    }
-}
